package nckd.poc602.plugin.operate;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.entity.plugin.AbstractOperationServicePlugIn;
import kd.bos.entity.plugin.PreparePropertysEventArgs;
import kd.bos.entity.plugin.args.BeforeOperationArgs;
import kd.bos.entity.plugin.args.BeginOperationTransactionArgs;
import kd.bos.entity.plugin.args.EndOperationTransactionArgs;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.servicehelper.operation.SaveServiceHelper;
import kd.sdk.plugin.Plugin;

/**
 * 付款处理提交校验超额&反写和撤销已付金额
 */
public class PayBillSubmitOperationServicePlugIn extends AbstractOperationServicePlugIn implements Plugin {
    @Override
    public void onPreparePropertys(PreparePropertysEventArgs e) {
        super.onPreparePropertys(e);
        e.getFieldKeys().add("entry");
        e.getFieldKeys().add("entry.nckd_refbillfield");
        e.getFieldKeys().add("entry.e_actamt");
    }
    
    @Override
    public void beforeExecuteOperationTransaction(BeforeOperationArgs e) {
    	// TODO Auto-generated method stub
    	super.beforeExecuteOperationTransaction(e);
    	DynamicObject[] payBills = e.getDataEntities();
    	String opkey = e.getOperationKey();
    	if("beforesubmit".equals(opkey)) {
    		this.updateSaleContractInfo(e, payBills, true);
    	}else if("unsubmit".equals(opkey) || "unaudit".equals(opkey)){
    		this.updateSaleContractInfo(e, payBills, false);
    	}
    }
    
    private void updateSaleContractInfo(BeforeOperationArgs e, DynamicObject[] payBills, Boolean iswriteback) {
		Set<DynamicObject> scs = new HashSet<>();
		List<Object> scsids = new ArrayList<>();
		Boolean existError = false;
		
		for(DynamicObject payBill : payBills) {
			DynamicObjectCollection entrys = payBill.getDynamicObjectCollection("entry");
			for(DynamicObject entry : entrys) {
				DynamicObject saleContract = entry.getDynamicObject("nckd_refbillfield");
				if(saleContract != null) {
					scsids.add(saleContract.getPkValue());
				}
			}
		}
		
		if(!scsids.isEmpty()) {
			QFilter qf = new QFilter("id",QCP.in,scsids);
			DynamicObject[] scsarray = BusinessDataServiceHelper.load("conm_salcontract", "id,receiptallamount,nckd_cgyfje,totalallamount,nckd_ycje", qf.toArray());
			for(DynamicObject payBill : payBills) {
				DynamicObjectCollection entrys = payBill.getDynamicObjectCollection("entry");
				String fbno = payBill.getString("billno");
				for(DynamicObject entry : entrys) {
					DynamicObject saleContract = entry.getDynamicObject("nckd_refbillfield");
					BigDecimal e_actamt = entry.getBigDecimal("e_actamt");//实付金额
					if(saleContract != null) {
						DynamicObject srsTemp = null;
						for(DynamicObject scsarrayitem : scsarray) {
							Object scsarrayitemid = scsarrayitem.getPkValue();
							if(scsarrayitemid.equals(saleContract.getPkValue())) {
								srsTemp = scsarrayitem;
								break;
							}
						}
						
						if(srsTemp != null) {
							BigDecimal jshj = srsTemp.getBigDecimal("totalallamount");//价税合计
							BigDecimal ycje = srsTemp.getBigDecimal("nckd_ycje");//已采金额
							BigDecimal receiptallamount = srsTemp.getBigDecimal("receiptallamount");//已收金额
							BigDecimal nckd_cgyfje = srsTemp.getBigDecimal("nckd_cgyfje");//采购已付金额
							if(iswriteback) {
								BigDecimal nckd_cgyfjeNew = e_actamt.add(nckd_cgyfje);
								if(nckd_cgyfjeNew.compareTo(receiptallamount) > 0) {
									existError = true;
									String error = "对应销售合同已收金额" + receiptallamount.setScale(2) +"，采购已付款金额" + nckd_cgyfje.setScale(2) + "，当前付款金额已超出可付金额";
									e.setCancel(true);
									e.setCancelMessage(error);
									break;
								}
								srsTemp.set("nckd_cgyfje", nckd_cgyfjeNew);
								payBill.set("nckd_ceyj", null);
								if(jshj.compareTo(BigDecimal.ZERO) != 0 && ycje.compareTo(BigDecimal.ZERO) != 0 ){
									//销售合同的比例：已收金额/价税合计 ;销售合同的比例：采购已付金额+当前付款金额/ 已采金额
									BigDecimal xsbl = receiptallamount.divide(jshj,2, BigDecimal.ROUND_HALF_UP);
									BigDecimal cgbl = (nckd_cgyfje.add(e_actamt)).divide(ycje,2, BigDecimal.ROUND_HALF_UP);
									if(cgbl.compareTo(xsbl) > 0) {
										String error = "对应销售合同收款比例" + xsbl.multiply(BigDecimal.valueOf(100)) +"%，采购已付款比例" + cgbl.multiply(BigDecimal.valueOf(100)) + "%，当前付款金额 已超出可付比例";
										payBill.set("nckd_ceyj", error);
									}else {
										String error = "1对应销售合同收款比例" + xsbl.multiply(BigDecimal.valueOf(100)) +"%，采购已付款比例" + cgbl.multiply(BigDecimal.valueOf(100)) + "%，当前付款金额 已超出可付比例";
										payBill.set("nckd_ceyj", error);
									}
								}
								scs.add(srsTemp);
							}else {
								BigDecimal nckd_cgyfjeNew = nckd_cgyfje.subtract(e_actamt);
								srsTemp.set("nckd_cgyfje", nckd_cgyfjeNew);
								scs.add(srsTemp);
								payBill.set("nckd_ceyj", null);
							}
						}
					}
				}
			}

			if(!existError && !scs.isEmpty()) {
				SaveServiceHelper.update(scs.toArray(new DynamicObject[0]));
			}
		}
    }
    
    @Override
    public void beginOperationTransaction(BeginOperationTransactionArgs e) {
    	// TODO Auto-generated method stub
    	super.beginOperationTransaction(e);
    }

    @Override
    public void endOperationTransaction(EndOperationTransactionArgs e) {
        super.endOperationTransaction(e);
    }
}